#include "user.h"
#include "system.h"
#include "LEDControl.h"
#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

void setupPWM(void)
{
        /*
     * PWM registers configuration
     * Fosc = 8000000 Hz
     * Fpwm = 500.00 Hz (Requested : 500 Hz)
     * Duty Cycle = 25 %
     * Resolution is 10 bits
     * Prescaler is 16
     * http://www.micro-examples.com/public/microex-navig/doc/097-pwm-calculator.html
     */
    CCP1CON = 0b00001111;
    T2CON = 0b00000111 ; // 16 as prescaler
    
    PR2 =  FOSC/PWM_FREQ/TMR2_PRESCALER/4 -1; // 61.5 or 111101
}

void setColor(HSV_t HSV)
{
    PSTRCON = HSV.hue; // set hue

    unsigned int temp = HSV.value * PR2 / 100;  //this is the value that needs to get written into CCPR1L:CCP1CON<5:4>

    // Check that we don't exceed 10bit capacity
    if (temp>1023)
    {
        temp = 1023;
    }
    
    DC1B0 = temp&0x01;          // mask 2 lower bits
    DC1B1 = (temp>>1)&0x01;
    CCPR1L = temp >> 2;         // assign 8 higher bits to ccpr1
}

void turnOffLED(void)
{
    PSTRCON = 0x00;
}