/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
#include "LEDControl.h"
/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */
uint8_t countSinceReset, countAlive, countDead, numNeighbors;
volatile extern bool StepFlag;

void InitApp(void)
{
    /* TODO Initialize User Ports/Peripherals/Project here */
    /* Setup analog functionality and port direction */
    currentState = false;

    TRISA =  0b00011011;
    TRISB = 0b11110000;
    TRISC = 0b11000011;

    ANSEL = 0x00;
    ANSELH = 0x00;
    
    /* Initialize peripherals */

    // ChangeCell - Interrupt-on-change
    IOCAbits.IOCA4 = 1;
    

    setupPWM(); 
    resetCell();

    /* Enable interrupts */
    INTCONbits.RABIE = 1;
    INTCONbits.GIE = 1;
}

void resetCell(void)
{
    /* Reset the cell*/
    bool resetON = true; // determine if on or off
    if (resetON){
        currentState = 1;

        HSV_t hsv;
        hsv.hue = WHITE;
        hsv.value = 50;
        setColor(hsv);
    }
    else{
        currentState = 0;
        turnOffLED();
    }
}

void stepCell(void) {
    static int hue = 0;
    
    HSV_t hsv;
    hsv.hue = hue;
    hsv.value = 50;
    
    hue = (hue + 1) % 8;

    setColor(hsv);
    /*countNeighbors();
    updateState();

    if (currentState){
        // we're alive!
        setCellColor();  // choose RGB based on algorithm

        countSinceReset += 1;
        countAlive +=1;
        countDead = 0;
        
        STATUS_PIN = 1;
    }else{
        // aww we died
        turnOffLED();

        countSinceReset += 1;
        countAlive = 0;
        countDead +=1;

        STATUS_PIN = 0;
    }*/
}

void countNeighbors(void)
{
    numNeighbors = PORTBbits.RB4 + \
                    PORTBbits.RB5 + \
                    PORTBbits.RB6 + \
                    PORTBbits.RB7 + \
                    PORTCbits.RC0 + \
                    PORTCbits.RC1 + \
                    PORTCbits.RC6 + \
                    PORTCbits.RC7;
}

void updateState(void)
{
    /* Conway's game of life */
    if(currentState == 1)
    {
        // Any live cell with fewer than two live neighbours dies, as if caused by under-population
        // Any live cell with more than three live neighbours dies, as if by overcrowding.
        if (numNeighbors<2 || numNeighbors > 3)
        {
            currentState = 0;
        }else{
            currentState = 1;
        }
    }else if(numNeighbors == 3)
    {
        //Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.
        currentState = 1;
    }
}

void setCellColor(void)
{
    /* Determine the color of the LED
     This could depend on:
     countSinceReset - lower value as further from reset
     countDead - if it has been a long time since alive, higher hue when alive again
     countAlive - hue decrease as cell has been alive longer */
#define MAX_STEPS 10

    HSV_t hsv;
    int temp = countDead/MAX_STEPS*7 + (1-countAlive/MAX_STEPS)*7;
    if (temp>7)
    {
        temp = 7;
    }else if (temp<0)
    {
        temp = 0;
    }
    hsv.hue = temp;
    hsv.value = countSinceReset*100/MAX_STEPS;

    setColor(hsv);  // set LED
    
}